runsimulation = true;

W=1;
H1=3;
H2=5;
T2=7;

LP=400;
HP=12000;
T_LPS=230;
T_HPS=560;
T_RH=560;

p=1800:3000;
if runsimulation
    % Connecting to HYSYS
    hysys=hyconnect([pwd '\steamturbinetest.HSC']);
    % Connecting to the 'I/O' spreadsheet in HYSYS
    sheet=hyspread(hysys,'I/O');
    % Waiting in case HYSYS is solving
    hywait(hysys)

    %Cell array for working with cells in the spreadsheet
    hc=cell(1,10);
    for i=1:10
        hc{i}=hycell(sheet,['A' num2str(i)]);
    end
    hc_LP=hycell(sheet,'B1');
    hc_HP=hycell(sheet,'B2');
    hc_LPStemp=hycell(sheet,'B3');
    hc_HPStemp=hycell(sheet,'B4');
    hc_reheattemp=hycell(sheet,'B5');
    hc_reheatenthalpy=hycell(sheet,'B6');
    
    hyset(hc_LP,LP);
    hyset(hc_HP,HP);
    hyset(hc_LPStemp,T_LPS);
    hyset(hc_HPStemp,T_HPS);
    hyset(hc_reheattemp,T_RH);
    
    % Writing to and reading from HYSYS
    restab=zeros(1201,8);
    Cp_RH=zeros(1201,1);
    for i=p
        hyset(hc{1},i)
        hywait(hysys)
        for j=3:10
            restab(i-1799,j-2)=hyvalue(hc{j});
        end
        Cp_RH(i-1799)=hyvalue(hc_reheatenthalpy);
    end
    
    hyrelease(hysys)
    Cp_RH=(Cp_RH-restab(:,H1+1))./(T_RH-restab(:,T2));
    save('Cpdata','restab', 'Cp_RH')
end

load Cpdata
Cp=zeros(length(restab),2);
a=zeros(2);
b=a;
aT=a;
bT=aT;

T1(1)=T_HPS;
T1(2)=T_RH;

for i=0:1
    Cp(:,1+i)=(restab(:,H1+i)-restab(:,H2+i))./(T1(1+i)-restab(:,T2+i));
    
    b(1+i,2)=((restab(1201,H1+i)-restab(1201,H2+i))-(restab(1,H1+i)-restab(1,H2+i)))./12;
    a(1+i,2)=mean(restab(:,H1+i)-restab(:,H2+i))-24*b(1+i,2);
    bT(1+i,2)=(restab(1201,T2+i)-restab(1,T2+i))./12;
    aT(1+i,2)=mean(restab(:,T2+i))-24*bT(1+i,2);
end
frmt='%30.27g\n';
fprintf(frmt,a(:,2))
fprintf(frmt,b(:,2))
fprintf(frmt,aT(:,2))
fprintf(frmt,bT(:,2))

h=figure;
plot(p/100,restab(:,W), 'r', p/100, restab(:,W+1),'g', p/100, (a(1,2)+p/100*b(1,2)), 'b', p/100, (a(2,2)+b(2,2)*p/100), 'c');
legend('HP (simulated)','IP (simulated)', 'HP (approximated)', ...
    'IP (approximated)', 'Location', 'Best')
xlabel('p [bar]')
ylabel('w [kJ/kg]')
saveas(h,'steamturbw','pdf')
saveas(h,'steamturbw','eps')

hT=figure;
plot(p/100,restab(:,T2), 'r', p/100, restab(:,T2+1),'g', p/100, (aT(1,2)+p/100*bT(1,2)), 'b', p/100, (aT(2,2)+bT(2,2)*p/100), 'c');
legend('HP (simulated)','IP (simulated)', 'HP (approximated)', ...
    'IP (approximated)', 'Location', 'Best')
xlabel('p [bar]')
ylabel('T [^{o}C]')
saveas(hT,'steamturbT','pdf')
saveas(hT,'steamturbT','eps')

[min(Cp_RH) mean(Cp_RH) max(Cp_RH)]
fprintf(frmt,mean(Cp_RH))

%plot(p,Cp(:,1),'b',p,Cp(:,2),'r')
%legend('HP', 'IP')