%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% bounds.m - setting upper and lower bounds for each variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used: none
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% 'Independent variables'
lowerbound(feedtemp)        = 200;  upperbound(feedtemp)        = 500;
lowerbound(LTStemp)         = 180;  upperbound(LTStemp)         = 250;
lowerbound(feedpressure)    =  18;  upperbound(feedpressure)    =  30;
lowerbound(HTStemp)         = 300;  upperbound(HTStemp)         = 450;
lowerbound(S2C)             =   1;  upperbound(S2C)             =   2;
lowerbound(airtemp)         = 410;  upperbound(airtemp)         = 530;
lowerbound(steamtemp)       = ...
    (T0(1,2)+lowerbound(feedpressure)*dTdp(1,2))*tempscale-C2K;
upperbound(steamtemp)       = ...
    (T0(1,2)+upperbound(feedpressure)*dTdp(1,2))*tempscale-C2K;

% 'Dependent variables'
lowerbound(CS_CO2)          =   0;  upperbound(CS_CO2) =CH4_in/M.CH4*M.CO2;
lowerbound(CS_CO)           =   0;  upperbound(CS_CO)  =CH4_in/M.CH4*M.CO;
lowerbound(CS_H2)           =   0;
upperbound(CS_H2)=CH4_in/M.CH4*M.H2*(2+upperbound(S2C));

lowerbound(HTStemp2) = lowerbound(LTStemp);   upperbound(HTStemp2) = 1000;
lowerbound(LTStemp2) = T_WRfeed;              upperbound(LTStemp2) = 500;

lowerbound(mCp_refprod)     = low(mCp_refprod);
upperbound(mCp_refprod)     = high(mCp_refprod);
lowerbound(mCp_HTSprod)     = low(mCp_HTSprod);
upperbound(mCp_HTSprod)     = high(mCp_HTSprod);
lowerbound(mCp_LTSprod)     = low(mCp_LTSprod);
upperbound(mCp_LTSprod)     = high(mCp_LTSprod);
lowerbound(mCp_GTfuel)      = low(mCp_GTfuel);
upperbound(mCp_GTfuel)      = high(mCp_GTfuel);

lowerbound(GTfueltemp)      = CCUoutlettemp;
upperbound(GTfueltemp) = T_fuel-C2K;

lowerbound(CCU_H2O)         =   0;
upperbound(CCU_H2O)=CH4_in/M.CH4*M.H2O*(2+upperbound(S2C));
lowerbound(CCU_CO)          =   0;  upperbound(CCU_CO) = CH4_in/M.CH4*M.CO;
lowerbound(CCU_CO2)         =   0;  upperbound(CCU_CO2)=CH4_in/M.CH4*M.CO2;
lowerbound(CCU_H2)          =   0;
upperbound(CCU_H2)=CH4_in/M.CH4*M.H2*(2+upperbound(S2C));
lowerbound(CCU_CH4)         =   0;  upperbound(CCU_CH4)=CH4_in*split.CH4(1);
lowerbound(CCU_N2)          =   0;
upperbound(CCU_N2)=CH4_in/M.CH4*M.O2/airO2mass*airN2mass*2;

lowerbound(fuelcompr)       =   0;
upperbound(fuelcompr) = CH4_in*LHV_CH4*.1;
    
% ATR inlet flow variables
lowerbound(feedflow) = airflow/CH4_in*.02;
upperbound(feedflow) = airflow/CH4_in*.05;

lowerbound(steamfeed)=lowerbound(S2C)*lowerbound(feedflow)*CH4_in*M.H2O/M.CH4;
upperbound(steamfeed)=upperbound(S2C)*upperbound(feedflow)*CH4_in*M.H2O/M.CH4;

% 'Combustion related variables'
lowerbound(exhaust_set)         = 0;
upperbound(exh_H2O)=airflow*airO2mass*.5*M.H2O*M.O2;
    %upperbound(feedflow)*CH4_in*(upperbound(S2C)+2)*M.H2O/M.CO2;
upperbound(exh_CO2)=upperbound(feedflow)*CH4_in*M.CO2/M.CH4;
upperbound(exh_O2)=airflow*airO2mass;
upperbound(exh_N2)=airflow*airN2mass+upperbound(feedflow)*CH4_in*(2)*M.O2/M.CO2/airO2mass;

% 'Air related variables'
lowerbound(comb_airflow)        = 0;
upperbound(comb_airflow)        = airflow;

% Turbo
lowerbound(W_GT)                = .2*lowerbound(feedflow)*CH4_in*LHV_CH4;
upperbound(W_GT)                = upperbound(feedflow)*CH4_in*LHV_CH4;
lowerbound(W_2ndcompr)          = 0;
upperbound(W_2ndcompr)          = upperbound(W_GT);
lowerbound(W_extcompr)          = 0;
upperbound(W_extcompr)          = upperbound(W_GT);
lowerbound(W_HPturb)            = 0;
upperbound(W_HPturb)            = upperbound(W_GT);
lowerbound(W_IPturb)            = 0;
upperbound(W_IPturb)            = upperbound(W_HPturb);

lowerbound(T_GT)                = 500+C2K;
upperbound(T_GT)                = 700+C2K;
    lowerbound(Ti_GT)               = 500+C2K;
    upperbound(Ti_GT)               = 600+C2K;
lowerbound(T_ATRair_2nd)        = T_air_2nd*tempscale;
lowerbound(T_ATRair_ext)        = T_air_ext*tempscale;
upperbound(T_ATRair_set)        = 600+C2K;
    lowerbound(Ti_ATRair_2nd)       = T_air_2nd*tempscale;
    lowerbound(Ti_ATRair_ext)       = T_air_ext*tempscale;
    upperbound(Ti_ATRair_set)       = 600+C2K;
lowerbound(T_HPturb)            = lowerbound(steamtemp)+C2K;
upperbound(T_HPturb)            = upperbound(steamtemp)+C2K;
    lowerbound(Ti_HPturb)           = 100+C2K;
    upperbound(Ti_HPturb)           = T_HP*tempscale;
lowerbound(T_IPturb)            = 100+C2K;
upperbound(T_IPturb)            = T_RH*tempscale;
    lowerbound(Ti_IPturb)           = 100+C2K;
    upperbound(Ti_IPturb)           = T_RH*tempscale;

lowerbound(p_GT)                = Outletpressure;
upperbound(p_GT)                = Outletpressure;
lowerbound(p_ATRair_set)        = lowerbound(feedpressure);
upperbound(p_ATRair_set)        = upperbound(feedpressure);
lowerbound(p_HPturb)            = lowerbound(feedpressure);
upperbound(p_HPturb)            = upperbound(feedpressure);
lowerbound(p_IPturb)            = lowerbound(feedpressure);
upperbound(p_IPturb)            = upperbound(feedpressure);

lowerbound(flow_ATRair_2nd)     = 0;
upperbound(flow_ATRair_2nd)     = ...
    upperbound(feedflow)*CH4_in/M.CH4*M.O2/airO2mass*2;
lowerbound(flow_ATRair_ext)     = 0;
upperbound(flow_ATRair_ext)     = ...
    upperbound(feedflow)*CH4_in/M.CH4*M.O2/airO2mass*2;
lowerbound(HPsteamflow)         = 0;
upperbound(HPsteamflow)         = ...
    upperbound(W_HPturb)/(w0(1,2)+upperbound(feedpressure)*dwdp(1,2));
lowerbound(IPsteamflow)         = 0;
upperbound(IPsteamflow)         = upperbound(HPsteamflow);

if aircompressorfix==1;
    upperbound(W_extcompr)=0;
    upperbound(flow_ATRair_ext)=0;
elseif aircompressorfix==2
    upperbound(W_2ndcompr)=0;
    upperbound(flow_ATRair_2nd)=0;
end

lowerbound(kappafrac_GT) =R/M.CO2/(HCp_coeffs.CO2*(TIT*tempscale).^(0:4)');
upperbound(kappafrac_GT) = R/M.N2/(HCp_coeffs.N2*lowerbound(T_GT).^(0:4)');
lowerbound(kappafrac_2ndcompr) = (airO2mass*R/M.O2+airN2mass*R/M.N2)/ ...
    (airO2mass*HCp_coeffs.O2*upperbound(Ti_ATRair_2nd).^(0:4)'+...
    airN2mass*HCp_coeffs.N2*upperbound(Ti_ATRair_2nd).^(0:4)');
upperbound(kappafrac_2ndcompr) = (airO2mass*R/M.O2+airN2mass*R/M.N2)/ ...
    (airO2mass*HCp_coeffs.O2*(T_air_2nd*tempscale).^(0:4)'+...
    airN2mass*HCp_coeffs.N2*(T_air_2nd*tempscale).^(0:4)');
lowerbound(kappafrac_extcompr) = lowerbound(kappafrac_2ndcompr);
upperbound(kappafrac_extcompr) = (airO2mass*R/M.O2+airN2mass*R/M.N2)/ ...
    (airO2mass*HCp_coeffs.O2*(T_air_ext*tempscale).^(0:4)' + ...
    airN2mass*HCp_coeffs.N2*(T_air_ext*tempscale).^(0:4)');
lowerbound(kappafrac_HPturb) =.1;
    %R/M.H2O/(HCp_coeffs.H2O*(T_HP*tempscale).^(0:4)');
upperbound(kappafrac_HPturb) =.5;
    %R/M.H2O/(HCp_coeffs.H2O*lowerbound(T_HPturb).^(0:4)');
lowerbound(kappafrac_IPturb) =.1;
    %R/M.H2O/(HCp_coeffs.H2O*(T_RH*tempscale).^(0:4)');
upperbound(kappafrac_IPturb) =.5;
    %R/M.H2O/(HCp_coeffs.H2O*lowerbound(T_IPturb).^(0:4)');

for i=feedflowdep(:,1)
    lowerbound(i)=lowerbound(i)*lowerbound(feedflow);
    upperbound(i)=upperbound(i)*upperbound(feedflow);
end

%TEMPSCALE
upperbound(outlettemp_set)=upperbound(outlettemp_set)./tempscale;
lowerbound(outlettemp_set)=lowerbound(outlettemp_set)./tempscale;
upperbound(idealoutlettemp_set)=upperbound(idealoutlettemp_set)./tempscale;
lowerbound(idealoutlettemp_set)=lowerbound(idealoutlettemp_set)./tempscale;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The following part is included mainly because there have been problems 
% with too wide bounds, and is meant to deal with solver stability problems
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Tightening bounds around the best solution found so far
if tighterbounds>0
    if tighterbounds==2
        load boundsfile
    end
    a=0;
    b=.1;
    for i=v_all'
        lowerbound(i)=max((1-a)*x_opt(i)+a*lowerbound(i), x_opt(i)/(1+b));
        upperbound(i)=min((1-a)*x_opt(i)+a*upperbound(i), x_opt(i)*(1+b));
    end
end
% Saving the bounds in case they need to be tightened
save('boundsfile', 'lowerbound', 'upperbound')