Parameters
         M1(streams,pc)          big M constant
         M2(streams,pc)          big M constant
         Tadj(streams)           constant for adjusting for HRAT;

Variables
         T_m(streams,pc)         (see t_M in Model chapter)
         Qp(iso,pc)              (see f_QP in Model chapter);

Binary variables
         y_hr(streams,pc);

Equations
         Q_H_eqn(pc)     Heating above pinch for every pinch candidate
         Q_balance       Total heat balance
         T_m1h(hot,pc)   these next equations are used to set T_M such that
         T_m2h(hot,pc)           Q_H_eqn accounts for only the heating and
         T_m3h(hot,pc)           cooling above pinch...
         T_m1c(cold,pc)
         T_m2c(cold,pc)
         T_m3c(cold,pc)
         Tisoh(isohot,pc)  likewise for isothermal streams...
         Tisoc(isocold,pc)
         Qpeqnh(isohot,pc)
         Qpeqn2h(isohot,pc)
         Qpeqnc(isocold,pc);

*Adjust cold streams with HRAT
Tadj(allcold)=HRAT;
Tadj(allhot)=0;

*Setting the big M constants
M1(streams,p)=1000;
M2(streams,p)=1000;
M1(ah,p) = T_in_up(p) + Tadj(p) - T_in_lo(ah) - Tadj(ah);
M2(h,p) = T_in_up(h) + Tadj(h) - T_in_lo(p) - Tadj(p);
M1(ac,p) = -T_in_lo(p) - Tadj(p) + T_in_up(ac) + Tadj(ac);
M2(c,p) = -T_in_lo(c) - Tadj(c) + T_in_up(p) + Tadj(p);
M2(i,p) = flow_up(i);
M1(s,p)$(M1(s,p)<0)=0;

Q_H_eqn(p) .. Q_H =g= sum(c, flow(c)*(T_out(c)-T_m(c,p)))
         - sum(h, flow(h)*(T_in(h)-T_m(h,p)))
         + sum(ic, Qp(ic,p)) - sum(ih, Qp(ih,p))
         ;
Q_balance .. Q_C - Q_H =e=
         sum(ni,(T_in(ni)-T_out(ni))*flow(ni))
         + sum(ih, flow(ih)) - sum(ic, flow(ic))
         ;

T_m1h(h,p) .. T_m(h,p) =g= T_out(h);
T_m2h(h,p) .. T_m(h,p) =g= T_in(p) + Tadj(p) - Tadj(h) - y_hr(h,p)*M1(h,p);
T_m3h(h,p) .. T_m(h,p) =g= T_in(h) - (1 - y_hr(h,p)) * M2(h,p);

T_m1c(c,p) .. T_m(c,p) =l= T_out(c);
T_m2c(c,p) .. T_m(c,p) =l= T_in(p) + Tadj(p) - Tadj(c) + (1-y_hr(c,p)) * M1(c,p);
T_m3c(c,p) .. T_m(c,p) =l= T_in(c) + y_hr(c,p) * M2(c,p);

*sametempchange(s) .. T_in(s) - T_out(s) =e= T_in_up(s) - T_out_up(s);

Tisoh(ih,p) .. T_in(ih) =g= T_in(p) + Tadj(p) - Tadj(ih) - y_hr(ih,p) * M1(ih,p) ;
Tisoc(ic,p) .. T_in(ic) =l= T_in(p) + Tadj(p) - Tadj(ic) + (1 - y_hr(ic,p)) * M1(ic,p);
Qpeqnh(ih,p) .. Qp(ih,p) =l= (1-y_hr(ih,p)) * M2(ih,p);
Qpeqn2h(ih,p) .. Qp(ih,p) =l= flow(ih);
Qpeqnc(ic,p) .. Qp(ic,p) =g= flow(ic) - y_hr(ic,p)*M2(ic,p);


Parameter ord2(streams)    (help parameter for the next equations);
ord2(streams)=ord(streams);

* Applying logic to fix binary variables
y_hr.fx(ah,p)$(ord2(ah) eq ord2(p))=1;
y_hr.fx(ac,p)$(ord2(ac) eq ord2(p))=0;
y_hr.fx(s,p)$(T_in_up(s)<T_in_lo(p)) = 1;
y_hr.fx(s,p)$(T_in_lo(s)>T_in_up(p)) = 0;

* Setting bounds for the 'constructed variables'
Qp.lo(i,p)=0;
Qp.up(i,p)=flow_up(i);
T_m.lo(h,p)=T_out_lo(h);
T_m.up(h,p)=max(T_in_up(h),T_in_up(p));
T_m.lo(c,p)=min(T_in_lo(c),T_in_lo(p));
T_m.up(c,p)=T_out_up(c);