%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% hr.m - setting and calculating parameters for heat integration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% GDXMRW:       writegdx.m      writing data to gdx file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Defining streams (index and name)
h1=length(uels)+1;      uels{h1}='h1';
h2=h1+1;                uels{h2}='h2';
h3=h2+1;                uels{h3}='h3';
h4=h3+1;                uels{h4}='exhaust';
c1=h4+1;                uels{c1}='c1';
LPE=c1+1;               uels{LPE}='LPE';
LPB=LPE+1;              uels{LPB}='LPB';
LPS=LPB+1;              uels{LPS}='LPS';
HPE=LPS+1;              uels{HPE}='HPE';
HPB=HPE+1;              uels{HPB}='HPB';
HPS=HPB+1;              uels{HPS}='HPS';
RH=HPS+1;               uels{RH}='RH';
preheat=RH+1;           uels{preheat}='preheat';
CCUreboiler=preheat+1;  uels{CCUreboiler}='CCUreboiler';
dummy=CCUreboiler+1;    uels{dummy}='dummy';

% Steam sets
hot=[h1 h2 h3 h4]';                     % Hot non-isothermal streams
cold=[c1 LPE LPS HPE HPS RH preheat]';  % Cold non-isothermal streams
isohot=[]';                             % Hot isothermal streams
isocold=[LPB HPB CCUreboiler]';         % Cold isothermal streams
iso=union(isohot', isocold')';          % All isothermal streams
noniso=union(hot', cold')';             % All non-isothermal streams
allhot=union(hot', isohot')';           % All hot streams
allcold=union(cold', isocold')';        % All cold streams
streams=union(iso', noniso')';          % All streams
st=length(streams);
%Set of pinch candidates
if relax==0
    pinchcandidates=[h1 h2 h3 h4 c1 LPE LPB HPE HPB RH preheat CCUreboiler]';
else
    pinchcandidates=[LPE CCUreboiler]';
end %LPE usually causes pinch (or CCUreboiler)

% Heat recovery approach temperature
HRAT=20;

% Limits for the exhaust outlet temperature
T_HRSG_out_lo=80;
T_HRSG_out_up=320;

% Given temperatures for the HRSG
T_water=32;
T_LPB=143.6;
T_HPB=324.6;
T_LPS=230;
T_HPS=560;
T_reheat=560;
T_CCUreboiler=120;

T_CH4supply=15;

% Calculating representative Cp values for the exhaust
T_exh1=630+C2K;
T_exh2=T_HRSG_out_lo+C2K;
Cp_H2O=HCp_coeffs.H2O./(1:5)*((T_exh1).^(1:5)'-(T_exh2).^(1:5)')/ ...
    (T_exh1-T_exh2);
Cp_CO2=HCp_coeffs.CO2./(1:5)*((T_exh1).^(1:5)'-(T_exh2).^(1:5)')/ ...
    (T_exh1-T_exh2);
Cp_O2=HCp_coeffs.O2./(1:5)*((T_exh1).^(1:5)'-(T_exh2).^(1:5)')/ ...
    (T_exh1-T_exh2);
Cp_N2=HCp_coeffs.N2./(1:5)*((T_exh1).^(1:5)'-(T_exh2).^(1:5)')/ ...
    (T_exh1-T_exh2);


Cp_RH=2.2318361545918672; %Calculated from HYSYS simulations
Cp_preheat=3.0748993810388652; %From HYSYS with feedtemp = 500 C

q_CCU=1500;
% Steam heating [kJ/h] per kg/h, calculated in HYSYS
q_LPE=470.418064578476;
q_LPB=2133.53929895280;
q_LPS=185.560334573640;
q_HPE=879.958181216821;
q_HPB=1197.71574220887;
q_HPS=816.054958821273;

w_LP=618.016839336527;
w_LPpump=.403;
w_HPpump=16.65;

if otherLP~=0
    if otherLP==8;
        T_LPB=170.4;
        T_LPS=230;
        q_LPE=586.4;
        q_LPB=2047;
        q_LPS=139.1;
        w_LP=696.2;
        w_LPpump=.9361;
    else
        disp(['Numbers for this pressure level LP = ' num2str(otherLP) ' not found.'])
    end
end
if otherHP~=0
    if otherHP==130;
        T_HPB=330.8;
        T_HPS=560;
        T_reheat=560;
        q_HPE=793.2;
        q_HPB=1135;
        q_HPS=828.3;
        Cp_RH=2.2381677011670842;
        if otherLP==8
            w_HPpump=18.14;
        else
            disp(['Numbers for HP = ' num2str(otherHP) ' not available for LP = ' num2str(otherLP)])
        end
    else
        disp(['Numbers for this pressure level HP= ' num2str(otherHP) ' not found.'])
    end
end


% Bounds on temperatures and flows
T_in_lo=[   h1      T_refprod;
            h2      lowerbound(HTStemp2);
            h3      lowerbound(LTStemp2);
            h4      lowerbound(T_GT)*tempscale-C2K;
            c1      lowerbound(GTfueltemp);
            LPE     T_water;
            LPB     T_LPB;
            LPS     T_LPB;
            HPE     T_LPB;
            HPB     T_HPB
            HPS     T_HPB;
            RH      lowerbound(steamtemp);
            preheat T_CH4supply;
            CCUreboiler T_CCUreboiler];
T_in_up=[   h1      T_refprod;
            h2      upperbound(HTStemp2);
            h3      upperbound(LTStemp2);
            h4      upperbound(T_GT)*tempscale-C2K;
            c1      upperbound(GTfueltemp);
            LPE     T_water;
            LPB     T_LPB;
            LPS     T_LPB;
            HPE     T_LPB;
            HPB     T_HPB;
            HPS     T_HPB;
            RH      upperbound(steamtemp);
            preheat T_CH4supply;
            CCUreboiler T_CCUreboiler];
T_out_lo=[  h1      lowerbound(HTStemp);
            h2      lowerbound(LTStemp);
            h3      T_WRfeed;
            h4      T_HRSG_out_lo;
            c1      T_fuel-C2K;
            LPE     T_LPB;
            LPS     T_LPS;
            HPE     T_HPB;
            HPS     T_HPS;
            RH      T_reheat;
            preheat lowerbound(feedtemp)];
T_out_up=[  h1      upperbound(HTStemp);
            h2      upperbound(LTStemp);
            h3      T_WRfeed;
            h4      T_HRSG_out_up;
            c1      T_fuel-C2K;
            LPE     T_LPB;
            LPS     T_LPS;
            HPE     T_HPB;
            HPS     T_HPS;
            RH      T_reheat;
            preheat upperbound(feedtemp)];
flow_lo=[   h1      lowerbound(mCp_refprod);
            h2      lowerbound(mCp_HTSprod);
            h3      lowerbound(mCp_LTSprod);
            h4      CH4_in*lowerbound(feedflow);%Not so good...
            c1      lowerbound(mCp_GTfuel);
            preheat lowerbound(feedflow)*CH4_in*Cp_preheat;
            CCUreboiler lowerbound(CCU_CO2)*split.CO2(2)*q_CCU];
flow_up=[   h1      upperbound(mCp_refprod);
            h2      upperbound(mCp_HTSprod);
            h3      upperbound(mCp_LTSprod);
            h4      airflow+CH4_in*(1+upperbound(S2C)*M.H2O/M.CH4+M.O2/M.CH4/airO2mass)*upperbound(feedflow);
            c1      upperbound(mCp_GTfuel);
            LPE     upperbound(W_GT);
            LPB     upperbound(W_GT);
            LPS     upperbound(W_GT);
            HPE     upperbound(W_GT);
            HPB     upperbound(W_GT);
            HPS     upperbound(W_GT);
            RH      upperbound(W_GT);
            preheat upperbound(feedflow)*CH4_in*Cp_preheat;
            CCUreboiler upperbound(CCU_CO2)*split.CO2(2)*q_CCU];

%Linking T_in variables with other temperature variables (or a constant)
T_inconn=[  h2  HTStemp2    1;
            h3  LTStemp2    1;
            h4  T_GT        tempscale;
            c1  GTfueltemp  1;
            RH  T_HPturb    tempscale];
T_inconnconst=[ h4          -C2K;
                LPE         T_water;
                LPB         T_LPB;
                LPS         T_LPB;
                HPE         T_LPB;
                HPB         T_HPB;
                HPS         T_HPB;
                RH          -C2K];
%Linking T_out variables with other temperature variables (or a constant)
T_outconn=[ h1  HTStemp     high(HTStemp)-low(HTStemp);
            h2  LTStemp     high(LTStemp)-low(LTStemp);
            preheat feedtemp high(feedtemp)-low(feedtemp)];
T_outconnconst=[h1          .5*(high(HTStemp)+low(HTStemp));
                h2          .5*(high(LTStemp)+low(LTStemp));
                LPE         T_LPB;
                LPS         T_LPS;
                HPE         T_HPB;
                HPS         T_HPS;
                RH          T_reheat;
                preheat     .5*(high(feedtemp)+low(feedtemp))];
%Linking flow variables from the heat recovery model with other flow
%variables
flowconn=[  h1      mCp_refprod 1;
            h2      mCp_HTSprod 1;
            h3      mCp_LTSprod 1;
            h4      exh_H2O     Cp_H2O;
            h4      exh_CO2     Cp_CO2;
            h4      exh_O2      Cp_O2;
            h4      exh_N2      Cp_N2;
            c1      mCp_GTfuel  1;
            HPE     HPsteamflow q_HPE/(T_HPB-T_LPB);
            RH      IPsteamflow Cp_RH;
            preheat feedflow    CH4_in*Cp_preheat;
            CCUreboiler CCU_CO2 split.CO2(2)*q_CCU];
%Linking flow variables from the heat recovery model with eachother
flowconn2=[ LPE LPB     q_LPE/(T_LPB-T_water)/q_LPB;
            LPE HPE     q_LPE/(T_LPB-T_water)/q_HPE*(T_HPB-T_LPB);
            LPB LPS     q_LPB/q_LPS*(T_LPS-T_LPB);
            HPE HPB     q_HPE/(T_HPB-T_LPB)/q_HPB;
            HPB HPS     q_HPB/q_HPS*(T_HPS-T_HPB)];
        
% Costs for utilities and streams
Q_Ccost=2.09793063613580e-3*objscale;
Q_Hcost=1*objscale;
flowcost=[  LPB -(w_LP*(1-STcondensercooling)-w_LPpump)/q_LPB;
            HPB (w_LPpump+w_HPpump)/q_HPB;
            RH  -w_LP*(1-STcondensercooling)/Cp_RH];
flowcost(:,2)=objscale*flowcost(:,2);

% Writing data to GDX file (hrsgconnected.gdx)
writegdx('hrsgconnected.gdx', ...
    'set',       'streams',     [streams; dummy], ...
    'set',       'hot',         [hot; dummy], ...
    'set',       'cold',        [cold; dummy], ...
    'set',       'noniso',      [noniso; dummy], ...
    'set',       'iso',         [iso; dummy], ...
    'set',       'isohot',      [isohot; dummy], ...
    'set',       'isocold',     [isocold; dummy], ...
    'set',       'allhot',      [allhot; dummy], ...
    'set',       'allcold',     [allcold; dummy], ...
    'set',       'pc',          [pinchcandidates; dummy], ...
    'parameter', 'T_in_lo',     T_in_lo, ...
    'parameter', 'T_in_up',     T_in_up, ...
    'parameter', 'T_out_lo',    T_out_lo, ...
    'parameter', 'T_out_up',    T_out_up, ...
    'parameter', 'flow_lo',     flow_lo, ...
    'parameter', 'flow_up',     flow_up, ...
    'parameter', 'T_inconn',    T_inconn, ...
    'parameter', 'T_outconn',   T_outconn, ...
    'parameter', 'flowconn',    flowconn, ...
    'parameter', 'flowconn2',   flowconn2, ...
    'parameter', 'T_outconnconst', T_outconnconst, ...
    'parameter', 'T_inconnconst', T_inconnconst, ...
    'parameter', 'HRAT',        HRAT, ...
    'parameter', 'Q_Hcost',     Q_Hcost, ...
    'parameter', 'Q_Ccost',     Q_Ccost, ...
    'parameter', 'flowcost',    flowcost, ...
    uels);