Set      streams                 set of all heat integration streams;
Sets     noniso(streams)         set of non-isothermal streams
         iso(streams)            set of isothermal streams
         cold(noniso)            set of cold non-isothermal streams
         hot(noniso)             set of hot non-isothermal streams
         isocold(iso)            set of cold isothermal streams
         isohot(iso)             set of hot isothermal streams
         allcold(streams)        set of all cold streams
         allhot(streams)         set of all hot streams
         pc(streams)             set of pinch candidate streams;

Parameters
         T_in_up(streams)        upper bound for inlet temperatures
         T_in_lo(streams)        lower bound for inlet temperatures
         T_out_up(streams)       upper bound for outlet temperatures
         T_out_lo(streams)       lower bound for outlet temperatures
         flow_lo(streams)        upper bound for flows (heat capacity or heat)
         flow_up(streams)        lower bound for flows (heat capacity or heat)
         HRAT                    heat recovery approach temperature
         Q_Hcost                 cost for hot utilities
         Q_Ccost                 cost for cold utilities
         flowcost(streams)       costs for streams
         T_inconn(streams,v_all) coefficients for connecting T_in variables
         T_outconn(streams,v_all) coefficients for connecting T_out variables
         flowconn(streams,v_all) coefficients for connecting flow variables
         flowconn2(streams,streams) coefficients for internally connecting flows
         T_inconnconst(streams)  (constant terms for T_in connections)
         T_outconnconst(streams) (constant terms for T_out connections);

* Reading data from GDX file
$GDXIN 'hrsgconnected.gdx'
$load streams, noniso, iso, cold, hot, isocold, isohot, allcold, allhot, pc
$load T_in_up, T_in_lo, T_out_up, T_out_lo
$load flow_up, flow_lo, HRAT, Q_Hcost, Q_Ccost, flowcost
$load T_inconn, T_outconn, flowconn, flowconn2, T_inconnconst, T_outconnconst
$GDXIN

* Defining subsets without the dummy stream
Sets     s(streams)
         h(hot)
         c(cold)
         ni(noniso)
         i(iso)
         ih(isohot)
         ic(isocold)
         p(pc)
         ah(allhot)
         ac(allcold);
s(streams)=yes;  s("dummy")=no;
h(hot)=yes;      h("dummy")=no;
c(cold)=yes;     c("dummy")=no;
ni(noniso)=yes;  ni("dummy")=no;
i(iso)=yes;      i("dummy")=no;
ih(isohot)=yes;  ih("dummy")=no;
ic(isocold)=yes; ic("dummy")=no;
p(pc)=yes;       p("dummy")=no;
ah(allhot)=yes;  ah("dummy")=no;
ac(allcold)=yes; ac("dummy")=no;

* Defining variables
Positive variables
         T_in(streams)   inlet (supply) temperatures
         T_out(streams)  outlet (target) temperatures
         flow            heat capacity or heat flows
         Q_H             hot utility usage
         Q_C             cold utility usage;

Equations
         T_inconnection(streams)    connecting T_in with other temperatures
         T_outconnection(streams)   connecting T_out with other temperatures
         flowconnection(streams)    connecting heat int. flows with other flows
         flowconnection2(streams)   connecting heat integration flows;

T_inconnection(s)$(sum(v_all, T_inconn(s,v_all)**2)>0 or T_inconnconst(s) <> 0)..
         T_in(s) =e= sum(v_all$(T_inconn(s,v_all) <> 0), T_inconn(s,v_all)*x(v_all)) + T_inconnconst(s);
T_outconnection(s)$(sum(v_all, T_outconn(s,v_all)**2)>0  or T_outconnconst(s) <> 0) ..
         T_out(s) =e= sum(v_all$(T_outconn(s,v_all) <> 0), T_outconn(s, v_all)*x(v_all)) + T_outconnconst(s);
flowconnection(s)$(sum(v_all, flowconn(s,v_all)**2)>0) ..
         flow(s) =e= sum(v_all$(flowconn(s,v_all) <> 0), flowconn(s, v_all)*x(v_all));
flowconnection2(s)$(sum(streams, flowconn2(s,streams)**2)>0) ..
         flow(s) =e= sum(streams$(flowconn2(s,streams) <> 0), flowconn2(s,streams)*flow(streams));

T_in.up(s)=T_in_up(s);
T_out.lo(s)=T_out_lo(s);
flow.up(s)=flow_up(s);

T_in.lo(s)=T_in_lo(s);
T_out.up(s)=T_out_up(s);
flow.lo(s)=flow_lo(s);

* Including the heat integration model
$include hr.gms
