%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% irccdata.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Including the gdxmrw package
addpath 'C:\Programfiler\GAMS23.6';
% Including the hysyslib toolbox
addpath 'C:\Users\Erik\Documents\MATLAB\hysyslib';

% The components present in the process
components={'CO' 'CO2' 'H2O' 'H2' 'CH4' 'O2' 'N2'};
% Molar mass of the components
M=struct('CO', 28.01, 'CO2', 44.01, 'H2O', 18.0153, 'H2', 2.0159,...
         'CH4', 16.042, 'O2', 31.9988, 'N2', 28.0134);
% Air composition
airO2=.21;              airN2=.79;
airO2mass=airO2*M.O2/(airO2*M.O2+airN2*M.N2);
airN2mass=airN2*M.N2/(airO2*M.O2+airN2*M.N2);
% Lower heating values
LHV_CO=10.9*1000;   % [kJ/kg]
LHV_H2=120.1*1000;  % [kJ/kg]
LHV_CH4=50.1*1000;  % [kJ/kg]
% CCU Splitting factors [H2 rich Syngas, CO2 to compression, Waste stream]
%   (Copy/paste from HYSYS)
split.CH4=[0.996000000000000 8.50000000000000e-004  3.14999999999999e-003];
split.H2O=[7.99000000000000e-002 0.916690000000000  3.41000000000002e-003];
split.CO= [0.997200000000000 4.40000000000000e-004	2.36000000000003e-003];
split.H2= [0.997100000000000 4.60000000000000e-004	2.44000000000000e-003];
split.CO2=[3.15000000000000e-002 0.960330000000000	8.17000000000001e-003];
split.N2= [0.998100000000000 2.20000000000000e-004	1.68000000000001e-003];
split.O2= [1.00000000000000  0.000000000000000      0.000000000000000];
% Flowrate of methane into the reformer
CH4_in=10000/flowscale;

% Some temperatures
T_refprod = 1000;       % Reformer product temperature [C]
T_WRfeed = 30;          % Water removal feed temperature [C]
CCUoutlettemp = 30;     % Temperature at the CCU outlet [C]
C2K = 273.15;           % Difference between Celsius and Kelvin scales

% Minimum capture ratio
mincapture=.9;

iv  = 7;                % Number of independent variables
dv  =17;                % Number of dependent variables
v   =iv+dv;
uels=cell(v,1);

% 'Independent variables'
feedtemp    =  1;       uels{feedtemp}    = 'feedtemp';
LTStemp     =  2;       uels{LTStemp}     = 'LTStemp';
feedpressure=  3;       uels{feedpressure}= 'feedpressure';
HTStemp     =  4;       uels{HTStemp}     = 'HTStemp';
S2C         =  5;       uels{S2C}         = 'S2C';
airtemp     =  6;       uels{airtemp}     = 'airtemp';
steamtemp   =  7;       uels{steamtemp}   = 'steamtemp';

% 'Dependent variables'
CS_CO2      =  1+iv;    uels{CS_CO2}      = 'CS_CO2';
CS_CO       =  2+iv;    uels{CS_CO}       = 'CS_CO';
CS_H2       =  3+iv;    uels{CS_H2}       = 'CS_H2';
HTStemp2    =  4+iv;    uels{HTStemp2}    = 'HTStemp2';
LTStemp2    =  5+iv;    uels{LTStemp2}    = 'LTStemp2';
mCp_GTfuel  =  6+iv;    uels{mCp_GTfuel}  = 'mCp_GTfuel';
mCp_HTSprod =  7+iv;    uels{mCp_HTSprod} = 'mCp_HTSprod';
mCp_LTSprod =  8+iv;    uels{mCp_LTSprod} = 'mCp_LTSprod';
mCp_refprod =  9+iv;    uels{mCp_refprod} = 'mCp_refprod';
GTfueltemp  = 10+iv;    uels{GTfueltemp}  = 'GTfueltemp';
CCU_H2O     = 11+iv;    uels{CCU_H2O}     = 'CCU_H2O';
CCU_CO      = 12+iv;    uels{CCU_CO}      = 'CCU_CO';
CCU_CO2     = 13+iv;    uels{CCU_CO2}     = 'CCU_CO2';
CCU_H2      = 14+iv;    uels{CCU_H2}      = 'CCU_H2';
CCU_CH4     = 15+iv;    uels{CCU_CH4}     = 'CCU_CH4';
CCU_N2      = 16+iv;    uels{CCU_N2}      = 'CCU_N2';
fuelcompr   = 17+iv;    uels{fuelcompr}   = 'fuelcompr';
regvarflows=[CS_CO2 CS_CO CS_H2 mCp_GTfuel mCp_HTSprod ...
    mCp_LTSprod mCp_refprod CCU_H2O CCU_CO CCU_CO2 CCU_H2 CCU_CH4 ...
    CCU_N2 fuelcompr];
feedflowdep=[CS_CO2 CS_CO CS_H2 mCp_GTfuel mCp_HTSprod mCp_LTSprod ...
    mCp_refprod CCU_H2O CCU_CO CCU_CO2 CCU_H2 CCU_CH4 CCU_N2 fuelcompr];
CCU_set=[CCU_H2O CCU_CO CCU_CO2 CCU_H2 CCU_CH4 CCU_N2];
v_set=(1:v)';
iv_set=(1:iv)';
dv_set=(1:dv)'+iv;

% Other variables (more are defined in combustion.m and air.m)
feedflow    = 1+iv+dv;  uels{feedflow}  = 'feedflow';

steamfeed   = 2+iv+dv;  uels{steamfeed} = 'steamfeed';
V=steamfeed;

% Combustion
exh_H2O     = 1+V;      uels{exh_H2O}   = 'exh_H2O';
exh_CO2     = 2+V;      uels{exh_CO2}   = 'exh_CO2';
exh_O2      = 3+V;      uels{exh_O2}    = 'exh_O2';
exh_N2      = 4+V;      uels{exh_N2}    = 'exh_N2';
exhaust_set=(exh_H2O:exh_N2)';
V=exh_N2;

% Turbo
T_GT        = 1+V;      uels{T_GT}       = 'T_GT';
T_ATRair_2nd= 2+V;      uels{T_ATRair_2nd}='T_ATRair_2nd';
T_ATRair_ext= 3+V;      uels{T_ATRair_ext}='T_ATRair_ext';
T_HPturb    = 4+V;      uels{T_HPturb}  = 'T_HPturb';
T_IPturb    = 5+V;      uels{T_IPturb}  = 'T_IPturb';
T_ATRair_set= [T_ATRair_2nd; T_ATRair_ext];
outlettemp_set=(T_GT:T_IPturb)';
V=T_IPturb;

W_GT       = 1+V;       uels{W_GT}      = 'W_GT';
W_2ndcompr  = 2+V;      uels{W_2ndcompr}= 'W_2ndcompr';
W_extcompr  = 3+V;      uels{W_extcompr}= 'W_extcompr';
W_HPturb    = 4+V;      uels{W_HPturb}  = 'W_HPturb';
W_IPturb    = 5+V;      uels{W_IPturb}  = 'W_IPturb';
W_set=(W_GT:W_IPturb)';
V=W_IPturb;

p_GT        = 1+V;      uels{p_GT}      = 'p_GT';
p_ATRair_2nd= 2+V;      uels{p_ATRair_2nd}='p_ATRair_2nd';
p_ATRair_ext= 3+V;      uels{p_ATRair_ext}='p_ATRair_ext';
p_HPturb    = 4+V;      uels{p_HPturb}  = 'p_HPturb';
p_IPturb    = 5+V;      uels{p_IPturb}  = 'p_IPturb';
p_ATRair_set= [p_ATRair_2nd; p_ATRair_ext];
p_out_set = (p_GT:p_IPturb)';
V=p_IPturb;

kappafrac_GT=1+V;    uels{kappafrac_GT}='kappafrac_GT';
kappafrac_2ndcompr=2+V;uels{kappafrac_2ndcompr}='kappafrac_2ndcompr';
kappafrac_extcompr=3+V; uels{kappafrac_extcompr}='kappafrac_extcompr';
kappafrac_HPturb=4+V;   uels{kappafrac_HPturb}='kappafrac_HPturb';
kappafrac_IPturb=5+V;   uels{kappafrac_IPturb}='kappafrac_IPturb';
kappafrac_set=(kappafrac_GT:kappafrac_IPturb)';
V=kappafrac_IPturb;

flow_ATRair_2nd=1+V;    uels{flow_ATRair_2nd}='flow_ATRair_2nd';
flow_ATRair_ext=2+V;    uels{flow_ATRair_ext}='flow_ATRair_ext';
flow_ATRair_set=[flow_ATRair_2nd; flow_ATRair_ext];
HPsteamflow =3+V;       uels{HPsteamflow}   = 'HPsteamflow';
IPsteamflow =4+V;       uels{IPsteamflow}   = 'IPsteamflow';
trbflows_set=[exhaust_set; flow_ATRair_set; HPsteamflow; IPsteamflow];
V=IPsteamflow;

comb_airflow= 1+V;       uels{comb_airflow}='comb_airflow';
V=comb_airflow;

Ti_GT        = 1+V;      uels{Ti_GT}      = 'Ti_GT';
Ti_ATRair_2nd=2+V;       uels{Ti_ATRair_2nd}='Ti_ATRair_2nd';
Ti_ATRair_ext= 3+V;      uels{Ti_ATRair_ext}='Ti_ATRair_ext';
Ti_HPturb    = 4+V;      uels{Ti_HPturb}  = 'Ti_HPturb';
Ti_IPturb    = 5+V;      uels{Ti_IPturb}  = 'Ti_IPturb';
Ti_ATRair_set= [Ti_ATRair_2nd; Ti_ATRair_ext];
idealoutlettemp_set=(Ti_GT:Ti_IPturb)';
V=Ti_IPturb;

v_all=(1:V)';

temperatures=[feedtemp LTStemp HTStemp airtemp steamtemp LTStemp2 ...
    HTStemp2 GTfueltemp T_GT T_ATRair_2nd T_ATRair_ext T_HPturb T_IPturb];
massflows=[CS_CO2 CS_CO CS_H2 CCU_H2 CCU_CO CCU_CO2 CCU_H2O CCU_N2 ...
    CCU_CH4 feedflow steamfeed exh_H2O exh_CO2 exh_O2 exh_N2 ...
    flow_ATRair_2nd flow_ATRair_ext HPsteamflow IPsteamflow comb_airflow];
energyflows=[mCp_GTfuel mCp_HTSprod mCp_LTSprod mCp_refprod W_GT ...
    W_extcompr W_2ndcompr W_HPturb W_IPturb fuelcompr];
pressures=[feedpressure p_GT p_ATRair_2nd p_ATRair_ext p_HPturb p_IPturb];