%REGCOEFFS fits second order polynomial curves to variables represented
%by the columns in A (containing simulation results), and writes the regression coefficients and other
%information to a .gdx file

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% Functions used:
%   "HOME MADE": 
%   - polyreg2.m
%   GDXMRW package:
%   - writegdx.m (using wgdx.m)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input variables which theoretically cannot impact the output variable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
noreg=zeros(v-iv, iv);
% HTS product temperature does not change as the LTS feed temperature
% is changed (because the latter is downstream of the former)
noreg(HTStemp2-iv,LTStemp)=1;
% Reformer product cooling does not change as the LTS feed temperature
% is changed (because the latter is downstream of the former)
noreg(mCp_refprod-iv,LTStemp)=1; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reading simulation data from file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load A_matrix7

vars=length(uels);

% Specifying the file to write to
filetowriteto='regcoeffs.gdx';

% Normalizing all values to the interval [-.5 , .5]
high=zeros(1,v);
low=high;
for i=1:v
    high(i)=max(A(:,i));
    low(i)=min(A(:,i));
    A(:,i)=(A(:,i)-.5*(high(i)+low(i)))./(high(i)-low(i));
end

% Fitting the data to a second order polynomial function of the dependent
% variables
nbeta=1+iv*(1.5+.5*iv);
if runregression
    y_hat=zeros(v-iv,nbeta);
    RSS=zeros(v-iv,1);
    for i=iv+1:v
        [y_hat(i-iv,:) RSS(i-iv,1)]=polyreg2(A(:,1:iv),A(:,i), noreg(i-iv,:));
    end
    save('regressioncoefficients','y_hat','RSS')
else
    load regressioncoefficients
end
 
% Writing the calculated regression coefficients to a .gmx-file
uels{vars+1}='beta_0';
i=1;
y_hat_gdx=zeros(nbeta*(v-iv),3);
for j=1:nbeta
    for k=1:dv
        y_hat_gdx(i,1)=j+vars;
        y_hat_gdx(i,2)=k+iv;
        y_hat_gdx(i,3)=y_hat(k,j);
        i=i+1;
    end
end
linbeta=zeros(iv,2);
quadbeta=linbeta;
for i=1:iv
    linbeta(i,1)=i;
    linbeta(i,2)=i+1+vars;
    quadbeta(i,1)=i;
    quadbeta(i,2)=i+1+iv+vars;
    uels{vars+1+i}=['beta_' num2str(i)];
    uels{vars+1+iv+i}=['beta_' num2str(i) ',' num2str(i)];
end
bilinbeta=zeros(.5*iv*(iv-1),3);
k=1;
for i=1:iv
    for j=i+1:iv
        bilinbeta(k,1)=i;
        bilinbeta(k,2)=j;
        bilinbeta(k,3)=k+1+2*iv+vars;
        uels{k+1+2*iv+vars}=['beta_' num2str(i) ',' num2str(j)];
        k=k+1;
    end
end
high_gdx=zeros(v,2);
low_gdx=high_gdx;
for i=1:v
    high_gdx(i,1)=i;
    high_gdx(i,2)=high(i);
    low_gdx(i,1)=i;
    low_gdx(i,2)=low(i);
end

b_set=(1:nbeta)'+vars;
writegdx(filetowriteto, ...
    'parameter', 'betas',   y_hat_gdx,...
    'set',       'v',       v_set, ...
    'set',       'iv',      iv_set, ...
    'set',       'dv',      dv_set, ...
    'set',       'b',       b_set, ...
    'set',       'lb',      linbeta, ...
    'set',       'qb',      quadbeta, ...
    'set',       'bb',      bilinbeta, ...
    'parameter', 'high',    high_gdx, ...
    'parameter', 'low',     low_gdx, ...
    uels);