% Connecting to HYSYS
hysys=hyconnect([pwd '\resultcheck7hr.HSC']);
% Connecting to the 'I/O' spreadsheet in HYSYS
sheet=hyspread(hysys,'I/O');
% Waiting in case HYSYS is solving
hywait(hysys)

%Cell array for working with cells in the spreadsheet
vars=55;
hc=cell(1,vars);
v_all=setdiff(v_all',idealoutlettemp_set')';
for i=v_all'
    hc{i}=hycell(sheet,['A' num2str(i)]);
end

% The variables to check
x_check=x_opt(v_all)';
x_check([feedpressure p_out_set'])=x_check([feedpressure p_out_set'])*100;
x_check(S2C)=x_check(steamfeed);
x_check(feedflow)=x_check(feedflow)*CH4_in;
x_check(outlettemp_set)=x_check(outlettemp_set)*tempscale-C2K;

% Sending values to HYSYS
freevars=[feedtemp LTStemp feedpressure HTStemp S2C airtemp feedflow ...
    p_ATRair_2nd p_ATRair_ext p_HPturb p_IPturb flow_ATRair_2nd ...
    flow_ATRair_ext HPsteamflow IPsteamflow comb_airflow steamtemp];
for j=freevars
    hyset(hc{j},x_check(j));
end

mflow_LPturb=x_opt(IPsteamflow)+mCp(LPB)/q_LPB;
T_LPturb=(x_opt(IPsteamflow)*(x_opt(T_IPturb)*tempscale-C2K)+mCp(LPB)/q_LPB*T_LPS)/mflow_LPturb;
sheet2=hyspread(hysys,'I/O_heatintegration');
hc2=cell(1,3);
for i=1:3
    hc2{i}=hycell(sheet2,['B' num2str(i)]);
end
hyset(hc2{1},mflow_LPturb)
hyset(hc2{2},T_LPturb)
hcT_exh_out=hycell(sheet2,'A10');
hcQ_exh=hycell(sheet2,'A1');
hcQ_LPB=hycell(sheet2,'A3');
hcLPflow=hycell(sheet2,'A13');
hcQ_LPE=hycell(sheet2,'A2');
hcSTflow=hycell(sheet2,'A15');
hcQ_CW=hycell(sheet2,'A11');
hcW_CW=hycell(sheet2,'A14');
hcQ_LPS=hycell(sheet2,'A4');
hcQ_HPB=hycell(sheet2,'A6');
hcQ_HPS=hycell(sheet2,'A7');
hcQ_RH=hycell(sheet2,'A8');
hcQ_preheat=hycell(sheet2,'A9');

hyset(hcT_exh_out,T_out(h4));
hyset(hcQ_LPB,mCp(LPB));
hyset(hcQ_LPE,mCp(LPE)*(T_out(LPE)-T_in(LPE)));
hyset(hcQ_CW,Q_C);

% Waiting for HYSYS to solve
hywait(hysys)

% Reading data from HYSYS spreadsheet cells
x_checked=zeros(vars,1);
for jj=v_all'
    try
        x_checked(jj)=hyvalue(hc{jj});
    catch ERR
        disp(['Could not find ' uels{jj} ' in HYSYS'])% - ' ERR.message])
    end
end
W_LPturb=hyvalue(hc2{3})/3600;
Q_exh=hyvalue(hcQ_exh)/3600;
LPflow=hyvalue(hcLPflow)/3600;
STflow=hyvalue(hcSTflow)/3600;
W_CW=hyvalue(hcW_CW)/3600;
Q_LPS=hyvalue(hcQ_LPS)/3600;
Q_HPB=hyvalue(hcQ_HPB)/3600;
Q_HPS=hyvalue(hcQ_HPS)/3600;
Q_RH=hyvalue(hcQ_RH)/3600;
Q_preheat=hyvalue(hcQ_preheat)/3600;

hyrelease(hysys);

flows=[S2C CS_CO2 CS_CO CS_H2 mCp_GTfuel mCp_HTSprod mCp_LTSprod ...
    mCp_refprod CCU_H2O CCU_CO CCU_CO2 CCU_H2 CCU_CH4 CCU_N2 fuelcompr ...
    feedflow steamfeed exh_H2O exh_CO2 exh_O2 exh_N2 W_GT W_2ndcompr ...
    W_extcompr W_HPturb W_IPturb flow_ATRair_2nd flow_ATRair_ext ...
    HPsteamflow IPsteamflow comb_airflow];
x_checked(flows)=x_checked(flows)/3600;
%x_checked(mCp_refprod)=x_checked(mCp_refprod)./(T_refprod - x_checked(HTStemp))*3600;
%x_checked(mCp_HTSprod)=x_checked(mCp_HTSprod)./(x_checked(HTStemp2) - x_checked(LTStemp))*3600;
%x_checked(mCp_LTSprod)=x_checked(mCp_LTSprod)./(x_checked(LTStemp2) - T_WRfeed)*3600;
%x_checked(mCp_GTfuel)=x_checked(mCp_GTfuel)./(200-x_checked(GTfueltemp))*3600;
x_check(mCp_refprod)=x_check(mCp_refprod).*(T_refprod - x_check(HTStemp));
x_check(mCp_HTSprod)=x_check(mCp_HTSprod).*(x_check(HTStemp2) - x_check(LTStemp));
x_check(mCp_LTSprod)=x_check(mCp_LTSprod).*(x_check(LTStemp2) - T_WRfeed);
x_check(mCp_GTfuel)=x_check(mCp_GTfuel).*(200-x_check(GTfueltemp));
x_checked(mCp_refprod)=x_checked(mCp_refprod).*3600;
x_checked(mCp_HTSprod)=x_checked(mCp_HTSprod).*3600;
x_checked(mCp_LTSprod)=x_checked(mCp_LTSprod).*3600;
x_checked(mCp_GTfuel)=x_checked(mCp_GTfuel).*3600;


x_checked(fuelcompr)=x_checked(fuelcompr)*3600;


x_check(massflows)=x_check(massflows)*flowscale/3600;
x_checked(massflows)=x_checked(massflows)*flowscale/3600;


x_error=x_check./x_checked-1;
frmt='%50s & %10.6g & %10.6g & %5.3f \\%% \\\\ \n';

v_report=setdiff(v_all',[iv_set' kappafrac_set' feedflow p_GT ...
    p_ATRair_2nd p_ATRair_ext flow_ATRair_ext flow_ATRair_2nd ...
    HPsteamflow IPsteamflow p_HPturb p_IPturb]);
variablenames
name=varname;
for i=v_report
    for j=1:length(name{i})
        if name{i}(j)=='_'
            name{i}(j)=' ';
        end
    end
    fprintf(frmt, name{i}, x_check(i), x_checked(i), x_error(i)*100)
end
x2_check=[mflow_LPturb*w_LP
        mCp(h4)*(T_in(h4)-T_out(h4))
        x_opt(HPsteamflow)
        Q_Ccost*Q_C_opt/objscale
        mCp(LPS)*(T_out(LPS)-T_in(LPS))
        mCp(HPB)
        mCp(HPS)*(T_out(HPS)-T_in(HPS))
        mCp(RH)*(T_out(RH)-T_in(RH))
        mCp(preheat)*(T_out(preheat)-T_in(preheat))];
x2_checked=[W_LPturb
        Q_exh
        STflow-LPflow
        W_CW
        Q_LPS
        Q_HPB
        Q_HPS
        Q_RH
        Q_preheat];
name2={'W_LPturb',
    'Q_exh',
    'HPflow',
    'W_CW',
    'Q_LPS',
    'Q_HPB',
    'Q_HPS',
    'Q_RH',
    'Q_preheat'};
name2={'LP steam turbine power & MW',
    'Exhaust cooling & MW',
    'HP steam flow & kg/s',
    'Cooling water pump work & MW',
    'LP steam superheating & MW',
    'HP steam boiling & MW',
    'HP steam superheating & MW',
    'IP steam reheating & MW',
    'Methane preheating & MW'};
x2_error=x2_check./x2_checked-1;
for i=1:length(x2_check)
    for j=1:length(name2{i})
        if name2{i}(j)=='_'
            name2{i}(j)=' ';
        end
    end
    fprintf(frmt, name2{i}, x2_check(i), x2_checked(i), x2_error(i)*100)
end


QC=x_checked(mCp_refprod)+x_checked(mCp_HTSprod)+x_checked(mCp_LTSprod)...
    +Q_exh-x_checked(mCp_GTfuel)-Q_LPS-Q_HPS-Q_HPB-Q_RH-Q_preheat+...
    Qflow(LPE)+Qflow(HPE)+Qflow(LPB)-...
    x_checked(CCU_CO2)*split.CO2(2)*q_CCU/1000;

W_net=(x_checked(W_GT) - W_air + ...
    (x_checked(W_HPturb)+x_checked(W_IPturb)+W_LPturb)*(1-STcondensercooling)-...
    x_checked(W_2ndcompr)-x_checked(W_extcompr)-x_checked(fuelcompr) - ...
    x_checked(CCU_CO2)*split.CO2(2)*w_CO2compr/1000-QC*Q_Ccost/objscale...
    -w_HPpump*(STflow-LPflow)-w_LPpump*STflow);
hysyseff=W_net/x_checked(feedflow)/LHV_CH4*1000*100