%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% setmodelparameters.m - setting some general equations and parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% GDXMRW:       writegdx.m      writing data to gdx file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Setting (some) linear inequality constraints
% (linconstr * x <= linconstrbound)
% Minimum 90 % CO2 captured:
    CO2captureconstr=1+length(uels);  uels{CO2captureconstr}='CO2capture';
linconstr_set=(CO2captureconstr:CO2captureconstr)';
linconstr=[ CO2captureconstr     CCU_CO2     -split.CO2(2)/M.CO2;
            CO2captureconstr     feedflow    CH4_in/M.CH4*mincapture];
linconstrbound=[CO2captureconstr            0];

% Setting (some) linear equality constraints (eqconstr * x = eqconstrconst)
% HP steam flow = LP steam flow + ATR steam flow:
    HPsteambalanceconstr=1+length(uels);
    uels{HPsteambalanceconstr}='HPsteambalance';
% Intermediate pressure = ATR pressure:
    IPconstr=1+HPsteambalanceconstr;     uels{IPconstr}='IP=feedpressure';
    IPconstr2=1+IPconstr;                uels{IPconstr2}='IP=p_HPturb_out';
% HP turbine outlet temperature = temperature of steam to ATR:
    steamtempconstr=1+IPconstr2; uels{steamtempconstr}='steam temperature';
eqconstr_set=(HPsteambalanceconstr:steamtempconstr)';
eqconstr=[HPsteambalanceconstr    HPsteamflow     1;
          HPsteambalanceconstr    IPsteamflow     -1;
          HPsteambalanceconstr    steamfeed       -1;
          IPconstr                feedpressure    1;
          IPconstr                p_HPturb        -1;
          IPconstr2               p_IPturb        1;
          IPconstr2               p_HPturb        -1;
          steamtempconstr         steamtemp       1;
          steamtempconstr         T_HPturb        -tempscale];
eqconstrconst= [HPsteambalanceconstr    0;
                IPconstr                0;
                IPconstr2               0;
                steamtempconstr         -C2K];
            
% Setting (some) product constraints (x1=k*x2*x3)
% Steam feed flow = methane feed flow * steam-to-carbon ratio * M.H2O/M.CH4
    steamfeedconstr=1+length(uels);     uels{steamfeedconstr}='Steam feed';
prodconstr_set=(steamfeedconstr:steamfeedconstr)';
prodconstr=[steamfeedconstr steamfeed S2C feedflow CH4_in*M.H2O/M.CH4];

% Dependent variables that are a function of the feed flow rate
feedflowdep=[feedflowdep' ones(length(feedflowdep), 1)];

% Writing data to file
writegdx('modelparameters.gdx', ...
    'set',       'lc',              linconstr_set, ...
    'set',       'ec',              eqconstr_set, ...
    'set',       'pr',              prodconstr_set, ...
    'set',       'feedflow',        feedflow, ...
    'parameter', 'feedflowdep',     feedflowdep, ...
    'parameter', 'linconstr',       linconstr, ...
    'parameter', 'linconstrbound',  linconstrbound, ...
    'parameter', 'eqconstr',        eqconstr, ...
    'parameter', 'eqconstrconst',   eqconstrconst, ...
    'parameter', 'prodconstr',      prodconstr, ...
    'parameter', 'startingpoint',   startingpoint, ...
    uels);