********************************************************************************
* turbo.gms
********************************************************************************

Sets     trb                             set of turbines
         trb1(trb)                       turbomachinery modeled physically
         trb2(trb)                       turbomach. modeled from simulations
         trbflows(v_all)                 turbomach. flow variables
         p_out(v_all)                    outlet pressure variables
         T_trb_out(v_all)                outlet temperature variables
         Ti_trb_out(v_all)               ideal outlet temperature variables
         W(v_all)                        power or work variables
         kappafrac(v_all)                (kappa-1) divided by kappa exponents
         exp                             set for temperature exponents;
Parameter
         eff(trb)                        turbine efficiencies
         turboflows(trb,trbflows)        which flows for which turbomachine
         Cpcoeffs(trbflows,exp)          specific heat equation coefficients
         T_trb_in(trb)                   inlet temperatures
         p_in(trb)                       inlet pressures
         kappafracsign(trb)              (used for changing p_in with p_out)
         R(trbflows)                     gas constants
         exponents(exp)                  exponents for specific heat eqn.
         w0(trb2)                        work for p_ATR=0 (constant term)
         dwdp(trb2)                      coefficient for pressure in work eqn.
         T0(trb2)                        temperature for p_ATR=0 (constant term)
         dTdp(trb2)                      coefficient for pressure in temp. eqn.;
$GDXIN turbo.gdx
$load trb,trb1,trb2,trbflows, p_out, T_trb_out,Ti_trb_out, W, kappafrac, exp
$load eff, turboflows, Cpcoeffs, T_trb_in, p_in, kappafracsign, R, exponents
$load w0, dwdp, T0, dTdp
$GDXIN

Equations
         turbine1(trb1, Ti_trb_out, W)                       power for trb1
         turbine2(trb2,W,p_feed)                             power for trb2
         idealturbinetemp(trb, Ti_trb_out, p_out, kappafrac) ideal outlet temp.s
         turbinetemp(trb1, T_trb_out, Ti_trb_out)            trb1 outlet temp.s
         turbinetemp2(trb2, T_trb_out, p_feed)               trb2 outlet temp.s
         comprexp(trb1,Ti_trb_out,kappafrac,W)     exponents in idealturbinetemp
;
*Variable deltaH(trb);

turbine1(trb1, Ti_trb_out, W)$(ord(Ti_trb_out) eq ord(trb1) and
         ord(W) eq ord(trb1)) .. x(W) =e= eff(trb1)*
         sum(trbflows$(turboflows(trb1,trbflows) eq 1), x(trbflows)*
         sum(exp, Cpcoeffs(trbflows,exp) *
         (T_trb_in(trb1)**exponents(exp)-x(Ti_trb_out)**exponents(exp)) /
         exponents(exp)));

turbine2(trb2,W,p_feed)$(ord(W) eq ord(trb2)+card(trb1)) .. x(W) =e=
         sum(trbflows$(turboflows(trb2,trbflows) eq 1), x(trbflows)*
         (w0(trb2) + dwdp(trb2)*(const(p_feed) + coeff(p_feed)*x(p_feed))));

idealturbinetemp(trb1,Ti_trb_out,p_out,kappafrac)$(ord(Ti_trb_out) eq ord(trb1)
         and ord(p_out) eq ord(trb1) and ord(kappafrac) eq ord(trb1)) ..
         x(Ti_trb_out) =e=
         T_trb_in(trb1) * (x(p_out)/p_in(trb1))**x(kappafrac);

turbinetemp(trb1,T_trb_out,Ti_trb_out)$(ord(T_trb_out) eq ord(trb1)
         and ord(Ti_trb_out) eq ord(trb1)) .. x(T_trb_out) =e=
         abs(eff(trb1))*x(Ti_trb_out)+(1-abs(eff(trb1)))*T_trb_in(trb1);
turbinetemp2(trb2, T_trb_out, p_feed)$(ord(T_trb_out) eq ord(trb2)+card(trb1))..
         x(T_trb_out) =e=
         T0(trb2) + dTdp(Trb2)*(const(p_feed)+coeff(p_feed)*x(p_feed));

comprexp(trb1,Ti_trb_out,kappafrac,W)$(ord(Ti_trb_out) eq ord(trb1) and
         ord(kappafrac) eq ord(trb1) and ord(W) eq ord (trb1)) ..
         eff(trb1) * sum(trbflows$(turboflows(trb1, trbflows) eq 1),
         x(trbflows)*R(trbflows)) * tempscale*(T_trb_in(trb1)-x(Ti_trb_out)) =e=
         x(kappafrac) * x(W);
